// Copyright (c) 2010 Keith D Gregory, all rights reserved
package com.kdgregory.app.s3util.dialogs;

import java.awt.Component;
import java.awt.event.ActionEvent;
import java.io.File;
import javax.swing.AbstractAction;
import javax.swing.BoxLayout;
import javax.swing.JCheckBox;
import javax.swing.JDialog;
import javax.swing.JFileChooser;
import javax.swing.JPanel;

import net.sf.swinglib.UIHelper;

import com.kdgregory.app.s3util.main.Concierge;


/**
 *  Manages a File Download dialog: a JFileChooser to select the destination
 *  directory, and some options to control the download.
 */
public class DownloadDialogController
{
    private Concierge _concierge;
    private File _selected;

    // the dialog is lazily constructed by show()
    private JDialog _dialog;
    private JFileChooser _fChooser;
    private JCheckBox _fFlatten;


    public DownloadDialogController(Concierge concierge)
    {
        _concierge = concierge;
    }


//----------------------------------------------------------------------------
//  Public Methods
//----------------------------------------------------------------------------

    /**
     *  Displays the (modal) dialog. If the user selects a file, returns it;
     *  otherwise returns <code>null</code>.
     */
    public File show()
    {
        if (_dialog == null)
            constructDialog();

        String prevDir = _concierge.getConfig().getLastDownloadDirectory();
        if (prevDir != null)
        {
            File prevAsFile = new File(prevDir);
            _fChooser.setCurrentDirectory(prevAsFile);
            _fChooser.setSelectedFile(prevAsFile);
        }

        _dialog.setVisible(true);
        return _selected;
    }


    /**
     *  Retrieves the selected file, <code>null</code> if the user clicked
     *  "Cancel".
     */
    public File getSelectedFile()
    {
        return _selected;
    }


    /**
     *  Retrieves the "flatten" checkbox value. Will be meaningless if
     *  the user selected a file or clicked "Cancel".
     */
    public boolean getFlatten()
    {
        return _fFlatten.isSelected();
    }


//----------------------------------------------------------------------------
//  Internals
//----------------------------------------------------------------------------

    private void constructDialog()
    {
        _dialog = UIHelper.newModalDialog(
                    _concierge.getDialogOwner(),
                    "Select Directory for Download",
                    createContentPane(),
                    new OKAction(), new CancelAction());
    }


    private JPanel createContentPane()
    {
        _fChooser = new JFileChooser();
        _fChooser.setFileHidingEnabled(false);
        _fChooser.setControlButtonsAreShown(false);
        _fChooser.setFileSelectionMode(JFileChooser.DIRECTORIES_ONLY);
        _fChooser.setMultiSelectionEnabled(false);
        _fChooser.setAlignmentX(Component.LEFT_ALIGNMENT);

        _fFlatten = new JCheckBox("Flatten directories");

        JPanel panel = new JPanel();
        panel.setLayout(new BoxLayout(panel, BoxLayout.Y_AXIS));
        panel.add(_fChooser);
        panel.add(_fFlatten);
        return panel;
    }


    private class OKAction
    extends AbstractAction
    {
        private static final long serialVersionUID = 1L;

        public OKAction()
        {
            super("Select");
        }

        public void actionPerformed(ActionEvent ignored)
        {
            _dialog.setVisible(false);
            _selected = _fChooser.getSelectedFile();
            if (_selected != null)
            {
                _concierge.getConfig()
                          .setLastDownloadDirectory(_selected.getAbsolutePath());
            }
        }
    }


    private class CancelAction
    extends AbstractAction
    {
        private static final long serialVersionUID = 1L;

        public CancelAction()
        {
            super("Cancel");
        }

        public void actionPerformed(ActionEvent ignored)
        {
            _selected = null;
            _dialog.setVisible(false);
        }
    }
}
