// Copyright (c) 2010 Keith D Gregory, all rights reserved
package com.kdgregory.app.s3util.dialogs;

import java.awt.Component;
import java.awt.FlowLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;
import java.io.File;
import javax.swing.AbstractAction;
import javax.swing.Action;
import javax.swing.BoxLayout;
import javax.swing.JCheckBox;
import javax.swing.JDialog;
import javax.swing.JFileChooser;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JTextField;

import net.sf.swinglib.UIHelper;

import com.kdgregory.app.s3util.main.Concierge;


/**
 *  Manages a File Upload dialog: a JFileChooser, a textfield for the
 *  destination, and some checkboxes to control what gets uploaded.
 */
public class UploadDialogController
{
    private Concierge _concierge;

    // the dialog is lazily constructed by show()
    private JDialog _dialog;
    private JFileChooser _fChooser;
    private JTextField _fSaveTo;
    private JCheckBox _fIsRecursive;
    private JCheckBox _fIsPublic;
    private Action _okAction;
    private Action _cancelAction;


    public UploadDialogController(Concierge concierge)
    {
        _concierge = concierge;
    }


//----------------------------------------------------------------------------
//  Public Methods
//----------------------------------------------------------------------------

    /**
     *  Displays the (modal) dialog. On return, indicates whether the user
     *  has selected any files (cancel clears the selection).
     */
    public boolean show()
    {
        if (_dialog == null)
            constructDialog();

        String prevDir = _concierge.getConfig().getLastUploadDirectory();
        if (prevDir != null)
        {
            _fChooser.setCurrentDirectory(new File(prevDir));
        }
        _fChooser.setSelectedFiles(null);
        _okAction.setEnabled(false);
        _dialog.setVisible(true);
        return getSelectedFiles().length > 0;
    }


    /**
     *  Retrieves the selected file, <code>null</code> if the user clicked
     *  "Cancel".
     */
    public File[] getSelectedFiles()
    {
        return _fChooser.getSelectedFiles();
    }


    /**
     *  Retrieves the destination directory name. Will be meaningless if
     *  the user clicked "Cancel".
     */
    public String getDestination()
    {
        return _fSaveTo.getText();
    }


    /**
     *  Returns the "make public" indicator.
     */
    public boolean getMakePublic()
    {
        return _fIsPublic.isSelected();
    }


//----------------------------------------------------------------------------
//  Internals
//----------------------------------------------------------------------------

    private void constructDialog()
    {
        _okAction = new OKAction();
        _cancelAction = new CancelAction();

        _dialog = UIHelper.newModalDialog(
                    _concierge.getDialogOwner(),
                    "Select File to Upload",
                    createContentPane(),
                    _okAction, _cancelAction);
    }


    private JPanel createContentPane()
    {
        _fChooser = new JFileChooser();
        _fChooser.setFileHidingEnabled(false);
        _fChooser.setControlButtonsAreShown(false);
        _fChooser.setFileSelectionMode(JFileChooser.FILES_AND_DIRECTORIES);
        _fChooser.setMultiSelectionEnabled(true);
        _fChooser.addPropertyChangeListener(new ChooserListener());
        _fChooser.setAlignmentX(Component.LEFT_ALIGNMENT);

        _fSaveTo = new JTextField("/", 30);
        JPanel pSaveTo = new JPanel(new FlowLayout(FlowLayout.LEFT));
        pSaveTo.setAlignmentX(Component.LEFT_ALIGNMENT);
        pSaveTo.add(new JLabel("Destination:"));
        pSaveTo.add(UIHelper.interButtonSpace());
        pSaveTo.add(_fSaveTo);

        _fIsRecursive = new JCheckBox("Recursively process directories");
        _fIsRecursive.setEnabled(true);
        _fIsRecursive.addActionListener(new IsRecursiveListener());

        _fIsPublic = new JCheckBox("Make Files Public");
        _fIsPublic.setEnabled(true);

        JPanel panel = new JPanel();
        panel.setLayout(new BoxLayout(panel, BoxLayout.Y_AXIS));
        panel.add(_fChooser);
        panel.add(pSaveTo);
        panel.add(_fIsRecursive);
        panel.add(_fIsPublic);
        return panel;
    }


    // this method is called whenever we change some state that would
    // result in a bad selection
    private void updateOKButton()
    {
        File selected = _fChooser.getSelectedFile();

        // first case is easy
        if (selected == null)
        {
            _okAction.setEnabled(false);
            return;
        }

        // don't allow directory selection unless doing recursive op
        if (selected.isDirectory())
        {
            _okAction.setEnabled(_fIsRecursive.isSelected());
            return;
        }

        // normal file selected, we can let user select it
        _okAction.setEnabled(true);
    }


    private class ChooserListener
    implements PropertyChangeListener
    {
        public void propertyChange(PropertyChangeEvent evt)
        {
            String prop = evt.getPropertyName();
            if (JFileChooser.DIRECTORY_CHANGED_PROPERTY.equals(prop))
            {
                _fChooser.setSelectedFile(null);
                File dir = _fChooser.getCurrentDirectory();
                if (dir != null)
                {
                    _concierge.getConfig()
                              .setLastUploadDirectory(dir.getAbsolutePath());
                }
            }
            else if (JFileChooser.SELECTED_FILE_CHANGED_PROPERTY.equals(prop))
            {
                updateOKButton();
            }
        }
    }


    // the OK button should not be enabled for directories unless the user
    // has checked this field; this listener is invoked on any change, so
    // that the button state can be re-evaluated
    private class IsRecursiveListener
    implements ActionListener
    {
        public void actionPerformed(ActionEvent e)
        {
            updateOKButton();
        }
    }


    // this action simply closes the dialog; all necessary data is held
    // in the dialog's fields
    private class OKAction
    extends AbstractAction
    {
        private static final long serialVersionUID = 1L;

        public OKAction()
        {
            super("Select");
        }

        public void actionPerformed(ActionEvent ignored)
        {
            _dialog.setVisible(false);
        }
    }


    private class CancelAction
    extends AbstractAction
    {
        private static final long serialVersionUID = 1L;

        public CancelAction()
        {
            super("Cancel");
        }

        public void actionPerformed(ActionEvent ignored)
        {
            _fChooser.setSelectedFiles(null);
            _dialog.setVisible(false);
        }
    }
}
