// Copyright (c) 2010 Keith D Gregory, all rights reserved
package com.kdgregory.app.s3util.filelist;

import java.util.ArrayList;
import java.util.Collections;
import java.util.Date;
import java.util.SortedSet;

import javax.swing.table.AbstractTableModel;



/**
 *  Holds the list of saved files.
 */
public class FileListTableModel
extends AbstractTableModel
{
    private static final long serialVersionUID = 1L;

    private ArrayList<S3File> _files;


    /**
     *  Constructs an empty instance.
     */
    public FileListTableModel()
    {
        _files = new ArrayList<S3File>();
    }


    /**
     *  Constructs an instance from an existing list of files.
     */
    public FileListTableModel(SortedSet<S3File> files)
    {
        _files = new ArrayList<S3File>(files);
    }


//----------------------------------------------------------------------------
//  TableModel
//----------------------------------------------------------------------------

    public int getColumnCount()
    {
        return 4;
    }


    public int getRowCount()
    {
        return _files.size();
    }


    @Override
    public String getColumnName(int col)
    {
        switch (col)
        {
            case 0 :
                return "Path";
            case 1 :
                return "Filename";
            case 2 :
                return "Size";
            case 3 :
                return "Uploaded";
            default :
                throw new IllegalArgumentException("invalid column: " + col);
        }
    }


    @Override
    public Class<?> getColumnClass(int col)
    {
        switch (col)
        {
            case 0 :
                return String.class;
            case 1 :
                return String.class;
            case 2 :
                return Integer.class;
            case 3 :
                return Date.class;
            default :
                throw new IllegalArgumentException("invalid column: " + col);
        }
    }


    public Object getValueAt(int row, int col)
    {
        switch (col)
        {
            case 0 :
                return getFileAt(row).getS3Path();
            case 1 :
                return getFileAt(row).getFile().getName();
            case 2 :
                return getFileAt(row).getContentLength();
            case 3 :
                return getFileAt(row).getLastModified();
            default :
                throw new IllegalArgumentException("invalid column: " + col);
        }
    }


//----------------------------------------------------------------------------
//  Public Methods
//----------------------------------------------------------------------------

    /**
     *  Returns the suggested relative widths of the columns in this model.
     */
    public int[] getRelativeWidths()
    {
        return new int[] { 50, 20, 15, 15 };
    }


    /**
     *  Returns the index of the named file, -1 if the file isn't in the
     *  model.
     */
    public int indexOf(S3File file)
    {
        int idx = Collections.binarySearch(_files, file);
        return (idx >= 0) ? idx : -1;
    }


    /**
     *  Returns the file at the specified index.
     */
    public S3File getFileAt(int row)
    {
        return _files.get(row);
    }


    /**
     *  Adds a new file to the model, inserting it at the proper location.
     *  If the file already exists in the model, it will be replaced (this
     *  may be used to update an entry's metadata).
     */
    public void addFile(S3File file)
    {
        int idx = _files.size();
        if ((idx == 0) || (file.compareTo(_files.get(idx - 1)) > 0))
        {
            _files.add(file);
            fireTableRowsInserted(idx, idx);
            return;
        }

        idx = Collections.binarySearch(_files, file);
        if (idx >= 0)
        {
            _files.set(idx, file);
            fireTableRowsUpdated(idx, idx);
        }
        else
        {
            // insert-at-end should have been caught above, so the corrected
            // index will be the element > file
            idx = -(idx + 1);
            _files.add(idx, file);
            fireTableRowsInserted(idx, idx);
        }
    }


    /**
     *  Removes a file from the model.
     */
    public void deleteFile(S3File file)
    {
        int idx = Collections.binarySearch(_files, file);
        if (idx < 0)
            return;

        _files.remove(idx);
        fireTableRowsDeleted(idx, idx);
    }


//----------------------------------------------------------------------------
//  Internals
//----------------------------------------------------------------------------

}
