// Copyright (c) 2010 Keith D Gregory, all rights reserved
package com.kdgregory.app.s3util.filelist;

import java.io.File;
import java.util.Date;

import org.apache.commons.lang.StringUtils;

import net.sf.practicalxml.XmlException;
import net.sf.practicalxml.XmlUtil;


/**
 *  An association between a local <code>File</code> and an S3 key, providing
 *  enough information to upload/download that file and display metadata about
 *  it.
 *  <p>
 *  For files to be uploaded to S3, the <code>file</code> component is an
 *  absolute path to a local file, while the <code>s3Path</code> component
 *  is a <em>relative</em> path from the root of the bucket. These are
 *  combined, with a slash between them if necessary, to produce the
 *  <code>key</code> component.
 *  <p>
 *  For files residing on S3, the <code>key</code> component is the absolute
 *  key used to retrieve the file. The <code>s3Path</code> component contains
 *  the initial substring of the key, up to but not including the final slash
 *  (if any). The <code>file</code> component is a simple filename, containing
 *  the substring of the key after the final slash (if any).
 *  <p>
 *  Note: S3 keys are <em>not</em> URL-encoded; the S34J library will handle
 *        encoding/decoding where needed.
 */
public final class S3File
implements Comparable<S3File>
{
    private File _file;
    private String _s3Path;
    private String _key;

    private Integer _contentLength;
    private Date _lastModified;


    /**
     *  Creates an instance from a local file, constructing the S3 key from
     *  provided destination path and local filename.
     *
     *  @param  file    The client-side filename. Used to access the physical
     *                  file during upload, and to get the file's name.
     *  @param  s3Path  The prefix that should be applied to the filename.
     */
    public S3File(File file, String s3Path)
    {
        _file = file.getAbsoluteFile();
        _s3Path = cleanupPath(s3Path);
        _key = makeKey(_s3Path, _file.getName());
    }


    /**
     *  Creates an instance from an S3 key; this will construct a local
     *  (relative) file.
     */
    public S3File(String key)
    {
        _key = key;
        _s3Path = extractPath(key);
        _file = extractFile(key);
    }


    /**
     *  Constructs an instance from an S3 key, along with metadata (note that
     *  the metadata is passed as strings, which is how it comes from S3).
     */
    public S3File(String key, String contentLength, String lastModified)
    {
        this(key);

        // both conversions ignore exceptions: I don't want to cause the
        // app to fail because of invalid metadata

        try
        {
            _contentLength = Integer.valueOf(contentLength);
        }
        catch (NumberFormatException ignored)
        {
            // ignored
        }

        try
        {
            _lastModified = XmlUtil.parseXsdDatetime(lastModified);
        }
        catch (XmlException ignored)
        {
            // ignored
        }
    }


//----------------------------------------------------------------------------
//  Accessor Methods
//----------------------------------------------------------------------------

    /**
     *  Returns the local file corresponding to this instance. For instances
     *  constructed locally, this will be an absolute pathname; for instances
     *  constructed from Amazon keys, it will be a single-component relative
     *  name.
     */
    public File getFile()
    {
        return _file;
    }


    /**
     *  Returns the S3 path component.
     */
    public String getS3Path()
    {
        return _s3Path;
    }


    /**
     *  Returns the key used to store this file on S3.
     */
    public String getKey()
    {
        return _key;
    }


    /**
     *  Returns the size of the file, as it is stored on S3. This is held
     *  as an <code>Integer</code> rather than an <code>int</code> because
     *  <code>JTableModel</code> deals in objects, not primitives.
     */
    public Integer getContentLength()
    {
        return _contentLength;
    }


    /**
     *  Returns the last modification date of the file (the last time it
     *  was uploaded to S3).
     */
    public Date getLastModified()
    {
        return _lastModified;
    }


//----------------------------------------------------------------------------
//  Object overrides, Comparable
//----------------------------------------------------------------------------

    /**
     *  The string value of this object is the key used to store it on S3.
     */
    @Override
    public String toString()
    {
        return _key;
    }


    /**
     *  Two instances are equal if they have the same S3 key.
     */
    @Override
    public boolean equals(Object obj)
    {
        if (this == obj)
            return true;
        else if (obj instanceof S3File)
        {
            return _key.equals(((S3File)obj)._key);
        }
        return false;
    }


    @Override
    public int hashCode()
    {
        return _key.hashCode();
    }


    public int compareTo(S3File that)
    {
        int ret = this._s3Path.compareTo(that._s3Path);
        if (ret != 0)
            return ret;

        String thisName = this._file.getName();
        String thatName = that._file.getName();
        return thisName.compareTo(thatName);
    }


//----------------------------------------------------------------------------
//  Internals
//----------------------------------------------------------------------------

    /**
     *  Given a string representing an S3 pathname, removes leading and
     *  trailing slashes.
     */
    private static String cleanupPath(String path)
    {
        if (path.startsWith("/"))
            path = path.substring(1);
        if (path.endsWith("/"))
            path = path.substring(0, path.length() - 1);
        return path;
    }


    /**
     *  Given a (possibly empty) path and filename, combines them into
     *  a key.
     */
    private static String makeKey(String path, String filename)
    {
        if (StringUtils.isEmpty(path))
            return filename;
        else
            return path + "/" + filename;
    }


    private static File extractFile(String key)
    {
        int sepIdx = key.lastIndexOf('/');
        return new File(key.substring(sepIdx + 1));
    }


    private static String extractPath(String key)
    {
        int sepIdx = key.lastIndexOf('/');
        if (sepIdx < 0)
            return "";
        else
            return cleanupPath(key.substring(0, sepIdx));
    }
}
