// Copyright (c) 2010 Keith D Gregory, all rights reserved
package com.kdgregory.app.s3util.main;

import java.util.prefs.Preferences;

import org.apache.commons.lang.StringUtils;


/**
 *  Provides bean-style access to application configuration data, backed by the
 *  Preferences API. Each preference has the following methods:
 *  <ul>
 *  <li> a getter, with the name <code>getXXX()</code>
 *  <li> a setter, with the name <code>setXXX()</code>
 *  <li> a method that returns validation regex, <code>xxxRegex()</code> (may
 *       be null, in which case no validation occurs)
 *  <li> a method that returns a description of the preference, <code>xxxInfo()</code>,
 *       used for tooltip text
 *  <ul>
 */
public class ConfigBean
{
    private final static String KEY_S3_ENDPOINT     = "S3Endpoint";
    private final static String KEY_S3_PUBLIC_KEY   = "S3PublicKey";
    private final static String KEY_S3_PRIVATE_KEY  = "S3PrivateKey";
    private final static String KEY_S3_BUCKET       = "S3BucketName";

    private final static String KEY_UPLOAD_DIR      = "LastUploadDirectory";
    private final static String KEY_DOWNLOAD_DIR    = "LastDownloadDirectory";


//----------------------------------------------------------------------------
//  Instance Data and Constructors
//----------------------------------------------------------------------------

    Preferences _prefs = Preferences.userNodeForPackage(this.getClass());


//----------------------------------------------------------------------------
//  Accessors
//----------------------------------------------------------------------------

    public static String lastDownloadDirectoryInfo()
    {
        return "The directory last used for a download operation.";
    }

    public static String lastDownloadDirectoryRegex()
    {
        return null;
    }


    public String getLastDownloadDirectory()
    {
        return _prefs.get(KEY_DOWNLOAD_DIR, null);
    }


    public void setLastDownloadDirectory(String name)
    {
        _prefs.put(KEY_DOWNLOAD_DIR, name);
    }


    public static String lastUploadDirectoryInfo()
    {
        return "The directory last used for an upload operation.";
    }

    public static String lastUploadDirectoryRegex()
    {
        return null;
    }


    public String getLastUploadDirectory()
    {
        return _prefs.get(KEY_UPLOAD_DIR, null);
    }


    public void setLastUploadDirectory(String name)
    {
        _prefs.put(KEY_UPLOAD_DIR, name);
    }


    public static String amazonBucketNameInfo()
    {
        return "The name of the bucket to be used to store backups. Multiple "
             + "backups can use the same bucket; we recommend one bucket per "
             + "machine being backed up, which holds all backups for that "
             + "machine.";
    }

    public static String amazonBucketNameRegex()
    {
        return "[-A-Za-z0-9]+";
    }


    public String getAmazonBucketName()
    {
        return _prefs.get(KEY_S3_BUCKET, "");
    }


    public void setAmazonBucketName(String name)
    {
        _prefs.put(KEY_S3_BUCKET, name);
    }


    public static String amazonPrivateKeyInfo()
    {
        return "The private key assigned to you by Amazon. This is used to "
             + "authenticate requests to S3, it is not used to encrypt the "
             + "data that you are backing up.";
    }

    public static String amazonPrivateKeyRegex()
    {
        return ".+";
    }

    public String getAmazonPrivateKey()
    {
        return _prefs.get(KEY_S3_PRIVATE_KEY, "");
    }

    public void setAmazonPrivateKey(String value)
    {
        _prefs.put(KEY_S3_PRIVATE_KEY, value);
    }


    public static String amazonPublicKeyInfo()
    {
        return "The public key assigned to you by Amazon. This is used to "
             + "authenticate requests to S3, it is not used to encrypt the "
             + "data that you are backing up.";
    }

    public static String amazonPublicKeyRegex()
    {
        return ".+";
    }

    public String getAmazonPublicKey()
    {
        return _prefs.get(KEY_S3_PUBLIC_KEY, "");
    }

    public void setAmazonPublicKey(String value)
    {
        _prefs.put(KEY_S3_PUBLIC_KEY, value);
    }


    public static String amazonEndpointInfo()
    {
        return "An Amazon-provided domain name that is appended to the "
             + "user-provided bucket name, to specify the host for S3 "
             + "requests.";
    }

    public static String amazonEndpointRegex()
    {
        return ".+\\.amazonaws\\.com";
    }

    public String getAmazonEndpoint()
    {
        return _prefs.get(KEY_S3_ENDPOINT, "s3.amazonaws.com");
    }

    public void setAmazonEndpoint(String value)
    {
        _prefs.put(KEY_S3_ENDPOINT, value);
    }


//----------------------------------------------------------------------------
//  Other Useful Functions
//----------------------------------------------------------------------------

    /**
     *  Determines whether we have sufficient prefs set to be able to
     *  do anything. This is used during application start, to decide
     *  whether to show the prefs window as the first action.
     */
    public boolean isConfigComplete()
    {
        return StringUtils.isNotEmpty(getAmazonEndpoint())
            && StringUtils.isNotEmpty(getAmazonPrivateKey())
            && StringUtils.isNotEmpty(getAmazonPublicKey());
    }
}
